@@ -365,6 +365,7 @@ class OfflineReader:
         self._keys = None
         self._best_reco = None
         self._header = None
+        self._usr = None
 
     def __getitem__(self, item):
         return OfflineReader(file_path=self._file_path, data=self._data[item])
@@ -379,7 +380,7 @@ class OfflineReader:
             if 'Head' in fobj:
                 self._header = {}
                 for n, x in fobj['Head']._map_3c_string_2c_string_3e_.items():
-                    self._header[n.decode("utf-8")] = x.decode("utf-8")
+                    self._header[n.decode("utf-8")] = x.decode("utf-8").strip()
             else:
                 warnings.warn("Your file header has an unsupported format")
         return self._header
@@ -474,6 +475,12 @@ class OfflineReader:
                 fitparameters=self.keys.fitparameters)
         return self._mc_tracks
 
+    @property
+    def usr(self):
+        if self._usr is None:
+            self._usr = Usr(self._file_path)
+        return self._usr
+
     @property
     def best_reco(self):
         """returns the best reconstructed track fit data. The best fit is defined
@@ -536,7 +543,7 @@ class OfflineReader:
              if j is not None])
         return max_reco_stages
 
-    def get_reco_fit(self, stages):
+    def get_reco_fit(self, stages, mc=False):
         """construct a numpy recarray of the fit information (reconstruction
         data) of the tracks reconstructed following the reconstruction stages
         of interest.
@@ -546,6 +553,10 @@ class OfflineReader:
         stages : list
             list of reconstruction stages of interest. for example
             [1, 2, 3, 4, 5].
+        mc : bool, optional
+            default is False to look for fit data in the tracks tree in offline files
+            (not the mc tracks tree). mc=True to look for fit data from the mc tracks
+            tree in offline files.
 
         Returns
         -------
@@ -560,24 +571,31 @@ class OfflineReader:
             are not found in the file.
         """
         keys = ", ".join(self.keys.fit_keys[:-1])
-        rec_stages = np.array(
-            [match for match in self._find_rec_stages(stages)])
+
+        if mc is False:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=False)])
+            fitinf = self.tracks.fitinf
+
+        if mc is True:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=True)])
+            fitinf = self.mc_tracks.fitinf
+
         mask = rec_stages[:, 1] != None
+
         if np.all(rec_stages[:, 1] == None):
             raise ValueError(
                 "The stages {} are not found in your file.".format(
                     str(stages)))
         else:
-            fit_data = np.array([
-                i[k]
-                for i, k in zip(self.tracks.fitinf[mask], rec_stages[:,
-                                                                     1][mask])
-            ])
+            fit_data = np.array(
+                [i[k] for i, k in zip(fitinf[mask], rec_stages[:, 1][mask])])
             rec_array = np.core.records.fromarrays(fit_data.transpose(),
                                                    names=keys)
             return rec_array
 
-    def get_reco_hits(self, stages, keys):
+    def get_reco_hits(self, stages, keys, mc=False):
         """construct a dictionary of hits class data based on the reconstruction
         stages of interest. For example, if the reconstruction stages of interest
         are [1, 2, 3, 4, 5], then get_reco_hits method will select the hits data 
@@ -591,6 +609,10 @@ class OfflineReader:
             [1, 2, 3, 4, 5].
         keys : list of str
             list of the hits class attributes.
+        mc : bool, optional
+            default is False to look for hits data in the hits tree in offline files
+            (not the mc_hits tree). mc=True to look for mc hits data in the mc hits
+            tree in offline files.
 
         Returns
         -------
@@ -604,19 +626,29 @@ class OfflineReader:
             are not found in the file.
         """
         lazy_d = {}
-        rec_stages = np.array(
-            [match for match in self._find_rec_stages(stages)])
+
+        if mc is False:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=False)])
+            hits_data = self.hits
+
+        if mc is True:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=True)])
+            hits_data = self.mc_hits
+
         mask = rec_stages[:, 1] != None
+
         if np.all(rec_stages[:, 1] == None):
             raise ValueError(
                 "The stages {} are not found in your file.".format(
                     str(stages)))
         else:
             for key in keys:
-                lazy_d[key] = getattr(self.hits, key)[mask]
+                lazy_d[key] = getattr(hits_data, key)[mask]
         return lazy_d
 
-    def get_reco_events(self, stages, keys):
+    def get_reco_events(self, stages, keys, mc=False):
         """construct a dictionary of events class data based on the reconstruction
         stages of interest. For example, if the reconstruction stages of interest
         are [1, 2, 3, 4, 5], then get_reco_events method will select the events data 
@@ -629,6 +661,10 @@ class OfflineReader:
             [1, 2, 3, 4, 5].
         keys : list of str
             list of the events class attributes.
+        mc : bool, optional
+            default is False to look for the reconstruction stages in the tracks tree
+            in offline files (not the mc tracks tree). mc=True to look for the reconstruction
+            data in the mc tracks tree in offline files.
 
         Returns
         -------
@@ -642,9 +678,17 @@ class OfflineReader:
             are not found in the file.
         """
         lazy_d = {}
-        rec_stages = np.array(
-            [match for match in self._find_rec_stages(stages)])
+
+        if mc is False:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=False)])
+
+        if mc is True:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=True)])
+
         mask = rec_stages[:, 1] != None
+
         if np.all(rec_stages[:, 1] == None):
             raise ValueError(
                 "The stages {} are not found in your file.".format(
@@ -654,7 +698,7 @@ class OfflineReader:
                 lazy_d[key] = getattr(self.events, key)[mask]
         return lazy_d
 
-    def get_reco_tracks(self, stages, keys):
+    def get_reco_tracks(self, stages, keys, mc=False):
         """construct a dictionary of tracks class data based on the reconstruction
         stages of interest. For example, if the reconstruction stages of interest
         are [1, 2, 3, 4, 5], then get_reco_tracks method will select tracks data 
@@ -668,6 +712,10 @@ class OfflineReader:
             [1, 2, 3, 4, 5].
         keys : list of str
             list of the tracks class attributes.
+        mc : bool, optional
+            default is False to look for tracks data in the tracks tree in offline files
+            (not the mc tracks tree). mc=True to look for tracks data in the mc tracks
+            tree in offline files.
 
         Returns
         -------
@@ -681,9 +729,19 @@ class OfflineReader:
             are not found in the file.
         """
         lazy_d = {}
-        rec_stages = np.array(
-            [match for match in self._find_rec_stages(stages)])
+
+        if mc is False:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=False)])
+            tracks_data = self.tracks
+
+        if mc is True:
+            rec_stages = np.array(
+                [match for match in self._find_rec_stages(stages, mc=True)])
+            tracks_data = self.mc_tracks
+
         mask = rec_stages[:, 1] != None
+
         if np.all(rec_stages[:, 1] == None):
             raise ValueError(
                 "The stages {} are not found in your file.".format(
@@ -692,13 +750,13 @@ class OfflineReader:
             for key in keys:
                 lazy_d[key] = np.array([
                     i[k] for i, k in zip(
-                        getattr(self.tracks, key)[mask], rec_stages[:,
+                        getattr(tracks_data, key)[mask], rec_stages[:,
                                                                     1][mask])
                 ])
 
         return lazy_d
 
-    def _find_rec_stages(self, stages):
+    def _find_rec_stages(self, stages, mc=False):
         """find the index of reconstruction stages of interest in a
         list of multiple reconstruction stages.
 
@@ -707,7 +765,10 @@ class OfflineReader:
         stages : list
             list of reconstruction stages of interest. for example
             [1, 2, 3, 4, 5].
-
+        mc : bool, optional
+            default is False to look for reconstruction stages in the tracks tree in
+            offline files (not the mc tracks tree). mc=True to look for reconstruction
+            stages in the mc tracks tree in offline files.
         Yields
         ------
         generator
@@ -715,7 +776,13 @@ class OfflineReader:
             interest if found. If the reconstruction stages of interest
             are not found, None is returned as the stages index.
         """
-        for trk_index, rec_stages in enumerate(self.tracks.rec_stages):
+        if mc is False:
+            stages_data = self.tracks.rec_stages
+
+        if mc is True:
+            stages_data = self.mc_tracks.rec_stages
+
+        for trk_index, rec_stages in enumerate(stages_data):
             try:
                 stages_index = rec_stages.index(stages)
             except ValueError:
@@ -756,6 +823,44 @@ class OfflineReader:
             yield i, j
 
 
+class Usr:
+    """Helper class to access AAObject usr stuff"""
+    def __init__(self, filepath):
+        self._f = uproot.open(filepath)
+        # Here, we assume that every event has the same names in the same order
+        # to massively increase the performance. This needs triple check if it's
+        # always the case; the usr-format is simply a very bad design.
+        try:
+            self._usr_names = [
+                n.decode("utf-8")
+                for n in self._f['E']['Evt']['usr_names'].array()[0]
+            ]
+        except (KeyError, IndexError):  # e.g. old aanet files
+            self._usr_names = []
+        else:
+            self._usr_idx_lookup = {
+                name: index
+                for index, name in enumerate(self._usr_names)
+            }
+            self._usr_data = self._f['E']['Evt']['usr'].lazyarray(
+                basketcache=uproot.cache.ThreadSafeArrayCache(
+                    BASKET_CACHE_SIZE))
+            for name in self._usr_names:
+                setattr(self, name, self[name])
+
+    def __getitem__(self, item):
+        return self._usr_data[:, self._usr_idx_lookup[item]]
+
+    def keys(self):
+        return self._usr_names
+
+    def __str__(self):
+        entries = []
+        for name in self.keys():
+            entries.append("{}: {}".format(name, self[name]))
+        return '\n'.join(entries)
+
+
 class OfflineEvents:
     """wrapper for offline events"""
     def __init__(self, keys, values):
@@ -910,12 +1015,8 @@ class OfflineTracks:
             setattr(self, k, v)
 
     def __getitem__(self, item):
-        if isinstance(item, int):
-            return OfflineTrack(self._keys, [v[item] for v in self._values],
-                                fitparameters=self._fitparameters)
-        else:
-            return OfflineTracks(self._keys, [v[item] for v in self._values],
-                                 fitparameters=self._fitparameters)
+        return OfflineTrack(self._keys, [v[item] for v in self._values],
+                            fitparameters=self._fitparameters)
 
     def __len__(self):
         try:
